/* --------------------------------------------------------
 ** Header file for awareness protocol parser functions.
 ** This functions are implemented in awparser.c.
 **
 ** Author : Manuele Kirsch Pinheiro
 ** Project CEMT - II/UFRGS - Opera Group/INRIA
 ** Finance by CNPq(Brazil)/INRIA(France) Cooperation
 **
 ** Date : 02 / Nov / 2001
 **
 ** $Id: awparser.h,v 1.2 2002/03/28 16:20:32 kirschpi Exp kirschpi $ 
 ** $Date: 2002/03/28 16:20:32 $
 ** $Author: kirschpi $
 ** $Revision: 1.2 $
 ** $Log: awparser.h,v $
 ** Revision 1.2  2002/03/28 16:20:32  kirschpi
 ** New function AwParser_searchInTree that searchs in the
 ** tree the value of an given element (thinking in XML
 ** elements, the value will be its child).
 **
 ** Revision 1.1  2001/11/02 16:32:17  kirschpi
 ** Initial revision
 **
 ** 
 * --------------------------------------------------------
 */

#ifndef AWPARSER_H
#define AWPARSER_H

#include <ctype.h>

#include "expat.h"
#include "awtree.h"

#define NUM_OF_NODES 3			/* estimative - number of children nodes per node */
#define XML_ELEMENT  1			/* identifier for nodes which contain XML elements */
#define TEXT_ELEMENT 2			/* identifier for nodes which contain text elements */

#ifndef MESSAGE_SIZE
#define MESSAGE_SIZE 2048		/* message protocol max size */
#endif


extern int verifyString (char *data);                                            	/* returns the number of non-space characters*/
extern AwNode *AwParser_makeNode (char *data, char **attr,int len, int type);    	/* creates a new node */
extern void AwParser_textElement (void *userData,const XML_Char *s,int len);          	/* parser text elements */
extern void AwParser_startElement(void *userData, const char *name, const char **attr);	/* parser start elements */
extern void AwParser_endElement(void *userData, const char *name);                     	/* parser end elements */
extern void AwParser_freeTree (AwTree *tree);                                         	/* deletes the tree and all its nodes */
extern void AwParser_freeNode (AwNode *n);                                             	/* deletes the node and its children */
extern AwString AwParser_mountMessage (AwTree *tree);                                 	/* mounts a protocol message */
extern AwString AwParser_mountElement (AwNode *node);                                  	/* mounts a xml string with the node data */
extern AwTree *AwParser_umountMessage (AwString message);                             	/* "umounts" a awareness protocol message */
extern char * AwParser_searchInTree (AwTree *tree, char *elem);                            	/* searches the value of an element in a XML tree */

#endif
